app.controller('productsController', function(
	$scope, $mdDialog, $mdToast, productsFactory, categoriesFactory, $q
){

	// read products
	$scope.read = function(json_url){

		// use products factory
		productsFactory.readProducts(json_url).then(function successCallback(response){
			$scope.products = response.data.records;
			$scope.paging = response.data.paging;
		}, function errorCallback(response){
			$scope.showToast("Unable to read record.");
		});

	}

	// show 'create product form' in dialog box
	$scope.showCreateProductForm = function(event){

		// retrieve categories list
		categoriesFactory.readAllCategories().then(function successCallback(response){

			$scope.categories = response.data.records;

			// show 'create product' dialog
			$mdDialog.show({
				controller: DialogController,
				templateUrl: './app/products/create_product.template.html',
				parent: angular.element(document.body),
				targetEvent: event,
				clickOutsideToClose: true,
				scope: $scope,
				preserveScope: true,
				fullscreen: true // Only for -xs, -sm breakpoints.
			});

		}, function errorCallback(response){
			$scope.showToast("Unable to read record.");
		});
	}

	// create new product
	$scope.createProduct = function(){

		productsFactory.createProduct($scope).then(function successCallback(response){

			// tell the user new product was created
			$scope.showToast(response.data.message);

			// refresh the list
			$scope.read('');

			// close dialog
			$scope.cancel();

			// remove form values
			$scope.clearProductForm();

		}, function errorCallback(response){
			$scope.showToast("Unable to create record.");
		});
	}

	// retrieve record to fill out the form
	$scope.readOneProduct = function(id){

		// get product to be edited
		productsFactory.readOneProduct(id).then(function successCallback(response){

			// put the values in form
			$scope.id = response.data.id;
			$scope.name = response.data.name;
			$scope.description = response.data.description;
			$scope.price = response.data.price;
			$scope.category_name = response.data.category_name;

			$mdDialog.show({
				controller: DialogController,
				templateUrl: './app/products/read_one_product.template.html',
				parent: angular.element(document.body),
				clickOutsideToClose: true,
				scope: $scope,
				preserveScope: true,
				fullscreen: true
			}).then(
				function(){},

				// user clicked 'Cancel'
				function() {
					// clear modal content
					$scope.clearProductForm();
				}
			);

		}, function errorCallback(response){
			$scope.showToast("Unable to retrieve record.");
		});

	}

	// retrieve record to fill out the form
	$scope.showUpdateProductForm = function(id){

		// use $q.all() to query required data before showing the 'update' form
		$q.all([
			categoriesFactory.readAllCategories(),
			productsFactory.readOneProduct(id)
		]).then(function(data){

			// categories
			$scope.categories = data[0].data.records;

			// product info
			$scope.id = data[1].data.id;
			$scope.name = data[1].data.name;
			$scope.description = data[1].data.description;
			$scope.price = data[1].data.price;
			$scope.category_id = data[1].data.category_id;

			// show dialog
			$mdDialog.show({
				controller: DialogController,
				templateUrl: './app/products/update_product.template.html',
				parent: angular.element(document.body),
				clickOutsideToClose: true,
				scope: $scope,
				preserveScope: true,
				fullscreen: true
			}).then(
				function(){},

				// user clicked 'Cancel'
				function() {
					// clear modal content
					$scope.clearProductForm();
				}
			);

		}, function errorCallback(response){
			$scope.showToast("Unable to load required data. " + response);
		});

		/*
		// retrieve categories list
		categoriesFactory.readAllCategories().then(function successCallback(response){

			$scope.categories = response.data.records;

		}, function errorCallback(response){
			$scope.showToast("Unable to read record.");
		});

		// get product to be edited
		productsFactory.readOneProduct(id).then(function successCallback(response){

			// put the values in form
			$scope.id = response.data.id;
			$scope.name = response.data.name;
			$scope.description = response.data.description;
			$scope.price = response.data.price;

			$mdDialog.show({
				controller: DialogController,
				templateUrl: './app/products/update_product.template.html',
				parent: angular.element(document.body),
				targetEvent: event,
				clickOutsideToClose: true,
				scope: $scope,
				preserveScope: true,
				fullscreen: true
			}).then(
				function(){},

				// user clicked 'Cancel'
				function() {
					// clear modal content
					$scope.clearProductForm();
				}
			);

		}, function errorCallback(response){
			$scope.showToast("Unable to retrieve record.");
		});
		*/
	}

	// update product record / save changes
	$scope.updateProduct = function(){

		productsFactory.updateProduct($scope).then(function successCallback(response){

			// tell the user product record was updated
			$scope.showToast(response.data.message);

			// refresh the product list
			$scope.read('');

			// close dialog
			$scope.cancel();

			// clear modal content
			$scope.clearProductForm();

		},
		function errorCallback(response) {
			$scope.showToast("Unable to update record.");
		});

	}

	// cofirm product deletion
	$scope.confirmDeleteProduct = function(event, id){

		// set id of record to delete
		$scope.id = id;

		// dialog settings
		var confirm = $mdDialog.confirm()
			.title('Are you sure?')
			.textContent('Product will be deleted.')
			.targetEvent(event)
			.ok('Yes')
			.cancel('No');

		// show dialog
		$mdDialog.show(confirm).then(
			// 'Yes' button
			function() {
				// if user clicked 'Yes', delete product record
				$scope.deleteProduct();
			},

			// 'No' button
			function() {
				// hide dialog
			}
		);
	}

	// delete product
	$scope.deleteProduct = function(){

		productsFactory.deleteProduct($scope.id).then(function successCallback(response){

			// tell the user product was deleted
			$scope.showToast(response.data.message);

			// refresh the list
			$scope.read('');

		}, function errorCallback(response){
			$scope.showToast("Unable to delete record.");
		});

	}

	// search products
	$scope.searchProducts = function(json_url){

		// use products factory
		productsFactory.searchProducts(json_url, $scope.product_search_keywords).then(function successCallback(response){
			$scope.products = response.data.records;
			$scope.paging = response.data.paging;
		}, function errorCallback(response){
			$scope.showToast("Unable to read records.");
		});
	}

	// clear variable / form values
	$scope.clearProductForm = function(){
		$scope.id = "";
		$scope.name = "";
		$scope.description = "";
		$scope.price = "";
	}

	// show toast message
	$scope.showToast = function(message){
		$mdToast.show(
			$mdToast.simple()
				.textContent(message)
				.hideDelay(3000)
				.position("top right")
		);
	}

	// methods for dialog box
	function DialogController($scope, $mdDialog) {
		$scope.cancel = function() {
			$mdDialog.cancel();
		};
	}

});
